@@ -20,6 +20,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 // Qt
 #include <QtTest/QtTest>
 #include <QImage>
+#include <QPainter>
 // KWin
 #include "../../src/client/compositor.h"
 #include "../../src/client/connection_thread.h"
@@ -51,7 +52,14 @@ private Q_SLOTS:
     void testMultipleSurfaces();
     void testOpaque();
     void testInput();
+    void testScale();
     void testDestroy();
+    void testUnmapOfNotMappedSurface();
+    void testDamageTracking();
+    void testSurfaceAt();
+    void testDestroyAttachedBuffer();
+    void testDestroyWithPendingCallback();
+    void testDisconnect();
 
 private:
     KWayland::Server::Display *m_display;
@@ -230,6 +238,7 @@ void TestWaylandSurface::testDamage()
     QVERIFY(serverSurface);
     QCOMPARE(serverSurface->damage(), QRegion());
     QVERIFY(serverSurface->parentResource());
+    QVERIFY(!serverSurface->isMapped());
 
     QSignalSpy damageSpy(serverSurface, SIGNAL(damaged(QRegion)));
     QVERIFY(damageSpy.isValid());
@@ -241,6 +250,7 @@ void TestWaylandSurface::testDamage()
     QCoreApplication::processEvents();
     QCoreApplication::processEvents();
     QVERIFY(damageSpy.isEmpty());
+    QVERIFY(!serverSurface->isMapped());
 
     QImage img(QSize(10, 10), QImage::Format_ARGB32);
     img.fill(Qt::black);
@@ -251,6 +261,7 @@ void TestWaylandSurface::testDamage()
     QVERIFY(damageSpy.wait());
     QCOMPARE(serverSurface->damage(), QRegion(0, 0, 10, 10));
     QCOMPARE(damageSpy.first().first().value<QRegion>(), QRegion(0, 0, 10, 10));
+    QVERIFY(serverSurface->isMapped());
 
     // damage multiple times
     QRegion testRegion(5, 8, 3, 6);
@@ -265,6 +276,7 @@ void TestWaylandSurface::testDamage()
     QVERIFY(damageSpy.wait());
     QCOMPARE(serverSurface->damage(), testRegion);
     QCOMPARE(damageSpy.first().first().value<QRegion>(), testRegion);
+    QVERIFY(serverSurface->isMapped());
 }
 
 void TestWaylandSurface::testFrameCallback()
@@ -407,6 +419,7 @@ void TestWaylandSurface::testAttachBuffer()
     QCOMPARE(serverSurface->buffer(), buffer3);
     QVERIFY(damageSpy.isEmpty());
     QVERIFY(unmappedSpy.isEmpty());
+    QVERIFY(serverSurface->isMapped());
 
     // clear the surface
     s->attachBuffer(blackBuffer);
@@ -419,6 +432,7 @@ void TestWaylandSurface::testAttachBuffer()
     QVERIFY(!unmappedSpy.isEmpty());
     QCOMPARE(unmappedSpy.count(), 1);
     QVERIFY(damageSpy.isEmpty());
+    QVERIFY(!serverSurface->isMapped());
 
     // TODO: add signal test on release
     buffer->unref();
@@ -631,6 +645,49 @@ void TestWaylandSurface::testInput()
     QCOMPARE(serverSurface->inputIsInfinite(), true);
 }
 
+void TestWaylandSurface::testScale()
+{
+    // this test verifies that updating the scale factor is correctly passed to the Wayland server
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    // create surface
+    QSignalSpy serverSurfaceCreated(m_compositorInterface, &CompositorInterface::surfaceCreated);
+    QVERIFY(serverSurfaceCreated.isValid());
+    QScopedPointer<Surface> s(m_compositor->createSurface());
+    QCOMPARE(s->scale(), 1);
+    QVERIFY(serverSurfaceCreated.wait());
+    SurfaceInterface *serverSurface = serverSurfaceCreated.first().first().value<KWayland::Server::SurfaceInterface*>();
+    QVERIFY(serverSurface);
+    QCOMPARE(serverSurface->scale(), 1);
+
+    // let's change the scale factor
+    QSignalSpy scaleChangedSpy(serverSurface, &SurfaceInterface::scaleChanged);
+    QVERIFY(scaleChangedSpy.isValid());
+    s->setScale(2);
+    QCOMPARE(s->scale(), 2);
+    // needs a commit
+    QVERIFY(!scaleChangedSpy.wait(100));
+    s->commit(Surface::CommitFlag::None);
+    QVERIFY(scaleChangedSpy.wait());
+    QCOMPARE(scaleChangedSpy.count(), 1);
+    QCOMPARE(scaleChangedSpy.first().first().toInt(), 2);
+    QCOMPARE(serverSurface->scale(), 2);
+
+    // let's try changing to same factor, should not emit changed on server
+    s->setScale(2);
+    s->commit(Surface::CommitFlag::None);
+    QVERIFY(!scaleChangedSpy.wait(100));
+
+    // but changing to a different value should still work
+    s->setScale(4);
+    s->commit(Surface::CommitFlag::None);
+    QVERIFY(scaleChangedSpy.wait());
+    QCOMPARE(scaleChangedSpy.count(), 2);
+    QCOMPARE(scaleChangedSpy.first().first().toInt(), 2);
+    QCOMPARE(scaleChangedSpy.last().first().toInt(), 4);
+    QCOMPARE(serverSurface->scale(), 4);
+}
+
 void TestWaylandSurface::testDestroy()
 {
     using namespace KWayland::Client;
@@ -656,5 +713,240 @@ void TestWaylandSurface::testDestroy()
     s->destroy();
 }
 
+void TestWaylandSurface::testUnmapOfNotMappedSurface()
+{
+    // this test verifies that a surface which doesn't have a buffer attached doesn't trigger the unmapped signal
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    // create surface
+    QSignalSpy serverSurfaceCreated(m_compositorInterface, &CompositorInterface::surfaceCreated);
+    QVERIFY(serverSurfaceCreated.isValid());
+    QScopedPointer<Surface> s(m_compositor->createSurface());
+    QVERIFY(serverSurfaceCreated.wait());
+    SurfaceInterface *serverSurface = serverSurfaceCreated.first().first().value<KWayland::Server::SurfaceInterface*>();
+
+    QSignalSpy unmappedSpy(serverSurface, &SurfaceInterface::unmapped);
+    QVERIFY(unmappedSpy.isValid());
+    QSignalSpy scaleChanged(serverSurface, &SurfaceInterface::scaleChanged);
+
+    // let's map a null buffer and change scale to trigger a signal we can wait for
+    s->attachBuffer(Buffer::Ptr());
+    s->setScale(2);
+    s->commit(Surface::CommitFlag::None);
+
+    QVERIFY(scaleChanged.wait());
+    QVERIFY(unmappedSpy.isEmpty());
+}
+
+void TestWaylandSurface::testDamageTracking()
+{
+    // this tests the damage tracking feature
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    // create surface
+    QSignalSpy serverSurfaceCreated(m_compositorInterface, &CompositorInterface::surfaceCreated);
+    QVERIFY(serverSurfaceCreated.isValid());
+    QScopedPointer<Surface> s(m_compositor->createSurface());
+    QVERIFY(serverSurfaceCreated.wait());
+    SurfaceInterface *serverSurface = serverSurfaceCreated.first().first().value<KWayland::Server::SurfaceInterface*>();
+
+    // before first commit, the tracked damage should be empty
+    QVERIFY(serverSurface->trackedDamage().isEmpty());
+
+    // Now let's damage the surface
+    QSignalSpy damagedSpy(serverSurface, &SurfaceInterface::damaged);
+    QImage image(QSize(100, 100), QImage::Format_ARGB32_Premultiplied);
+    image.fill(Qt::red);
+    s->attachBuffer(m_shm->createBuffer(image));
+    s->damage(QRect(0, 0, 100, 100));
+    s->commit(Surface::CommitFlag::None);
+    QVERIFY(damagedSpy.wait());
+    QCOMPARE(serverSurface->trackedDamage(), QRegion(0, 0, 100, 100));
+    QCOMPARE(serverSurface->damage(), QRegion(0, 0, 100, 100));
+
+    // resetting the tracked damage should empty it
+    serverSurface->resetTrackedDamage();
+    QVERIFY(serverSurface->trackedDamage().isEmpty());
+    // but not affect the actual damage
+    QCOMPARE(serverSurface->damage(), QRegion(0, 0, 100, 100));
+
+    // let's damage some parts of the surface
+    QPainter p;
+    p.begin(&image);
+    p.fillRect(QRect(0, 0, 10, 10), Qt::blue);
+    p.end();
+    s->attachBuffer(m_shm->createBuffer(image));
+    s->damage(QRect(0, 0, 10, 10));
+    s->commit(Surface::CommitFlag::None);
+    QVERIFY(damagedSpy.wait());
+    QCOMPARE(serverSurface->trackedDamage(), QRegion(0, 0, 10, 10));
+    QCOMPARE(serverSurface->damage(), QRegion(0, 0, 10, 10));
+
+    // and damage some part completely not bounding to the current damage region
+    p.begin(&image);
+    p.fillRect(QRect(50, 40, 20, 30), Qt::blue);
+    p.end();
+    s->attachBuffer(m_shm->createBuffer(image));
+    s->damage(QRect(50, 40, 20, 30));
+    s->commit(Surface::CommitFlag::None);
+    QVERIFY(damagedSpy.wait());
+    QCOMPARE(serverSurface->trackedDamage(), QRegion(0, 0, 10, 10).united(QRegion(50, 40, 20, 30)));
+    QCOMPARE(serverSurface->trackedDamage().rectCount(), 2);
+    QCOMPARE(serverSurface->damage(), QRegion(50, 40, 20, 30));
+
+    // now let's reset the tracked damage again
+    serverSurface->resetTrackedDamage();
+    QVERIFY(serverSurface->trackedDamage().isEmpty());
+    // but not affect the actual damage
+    QCOMPARE(serverSurface->damage(), QRegion(50, 40, 20, 30));
+}
+
+void TestWaylandSurface::testSurfaceAt()
+{
+    // this test verifies that surfaceAt(const QPointF&) works as expected for the case of no children
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    // create surface
+    QSignalSpy serverSurfaceCreated(m_compositorInterface, &CompositorInterface::surfaceCreated);
+    QVERIFY(serverSurfaceCreated.isValid());
+    QScopedPointer<Surface> s(m_compositor->createSurface());
+    QVERIFY(serverSurfaceCreated.wait());
+    SurfaceInterface *serverSurface = serverSurfaceCreated.first().first().value<KWayland::Server::SurfaceInterface*>();
+
+    // a newly created surface should not be mapped and not provide a surface at a position
+    QVERIFY(!serverSurface->isMapped());
+    QVERIFY(!serverSurface->surfaceAt(QPointF(0, 0)));
+
+    // let's damage this surface
+    QSignalSpy sizeChangedSpy(serverSurface, &SurfaceInterface::sizeChanged);
+    QVERIFY(sizeChangedSpy.isValid());
+    QImage image(QSize(100, 100), QImage::Format_ARGB32_Premultiplied);
+    image.fill(Qt::red);
+    s->attachBuffer(m_shm->createBuffer(image));
+    s->damage(QRect(0, 0, 100, 100));
+    s->commit(Surface::CommitFlag::None);
+    QVERIFY(sizeChangedSpy.wait());
+
+    // now the surface is mapped and surfaceAt should give the surface
+    QVERIFY(serverSurface->isMapped());
+    QCOMPARE(serverSurface->surfaceAt(QPointF(0, 0)), serverSurface);
+    QCOMPARE(serverSurface->surfaceAt(QPointF(100, 100)), serverSurface);
+    // outside the geometry it should not give a surface
+    QVERIFY(!serverSurface->surfaceAt(QPointF(101, 101)));
+    QVERIFY(!serverSurface->surfaceAt(QPointF(-1, -1)));
+}
+
+void TestWaylandSurface::testDestroyAttachedBuffer()
+{
+    // this test verifies that destroying of a buffer attached to a surface works
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    // create surface
+    QSignalSpy serverSurfaceCreated(m_compositorInterface, &CompositorInterface::surfaceCreated);
+    QVERIFY(serverSurfaceCreated.isValid());
+    QScopedPointer<Surface> s(m_compositor->createSurface());
+    QVERIFY(serverSurfaceCreated.wait());
+    SurfaceInterface *serverSurface = serverSurfaceCreated.first().first().value<KWayland::Server::SurfaceInterface*>();
+
+    // let's damage this surface
+    QSignalSpy damagedSpy(serverSurface, &SurfaceInterface::damaged);
+    QVERIFY(damagedSpy.isValid());
+    QImage image(QSize(100, 100), QImage::Format_ARGB32);
+    image.fill(Qt::red);
+    s->attachBuffer(m_shm->createBuffer(image));
+    s->damage(QRect(0, 0, 100, 100));
+    s->commit(Surface::CommitFlag::None);
+    QVERIFY(damagedSpy.wait());
+    QVERIFY(serverSurface->buffer());
+
+    // attach another buffer
+    image.fill(Qt::blue);
+    s->attachBuffer(m_shm->createBuffer(image));
+    m_connection->flush();
+
+    // Let's try to destroy it
+    QSignalSpy destroySpy(serverSurface->buffer(), &BufferInterface::aboutToBeDestroyed);
+    QVERIFY(destroySpy.isValid());
+    delete m_shm;
+    m_shm = nullptr;
+    QVERIFY(destroySpy.wait());
+
+    // TODO: should this emit unmapped?
+    QVERIFY(!serverSurface->buffer());
+}
+
+
+void TestWaylandSurface::testDestroyWithPendingCallback()
+{
+    // this test tries to verify that destroying a surface with a pending callback works correctly
+    // first create surface
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    QScopedPointer<Surface> s(m_compositor->createSurface());
+    QVERIFY(!s.isNull());
+    QVERIFY(s->isValid());
+    QSignalSpy surfaceCreatedSpy(m_compositorInterface, &CompositorInterface::surfaceCreated);
+    QVERIFY(surfaceCreatedSpy.isValid());
+    QVERIFY(surfaceCreatedSpy.wait());
+    auto serverSurface = surfaceCreatedSpy.first().first().value<SurfaceInterface*>();
+    QVERIFY(serverSurface);
+
+    // now render to it
+    QImage img(QSize(10, 10), QImage::Format_ARGB32);
+    img.fill(Qt::black);
+    auto b = m_shm->createBuffer(img);
+    s->attachBuffer(b);
+    s->damage(QRect(0, 0, 10, 10));
+    // add some frame callbacks
+    for (int i = 0; i < 1000; i++) {
+        wl_surface_frame(*s);
+    }
+    s->commit(KWayland::Client::Surface::CommitFlag::FrameCallback);
+    QSignalSpy damagedSpy(serverSurface, &SurfaceInterface::damaged);
+    QVERIFY(damagedSpy.isValid());
+    QVERIFY(damagedSpy.wait());
+
+    // now try to destroy the Surface again
+    QSignalSpy destroyedSpy(serverSurface, &QObject::destroyed);
+    QVERIFY(destroyedSpy.isValid());
+    s.reset();
+    QVERIFY(destroyedSpy.wait());
+}
+
+void TestWaylandSurface::testDisconnect()
+{
+    // this test verifies that the server side correctly tears down the resources when the client disconnects
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    QScopedPointer<Surface> s(m_compositor->createSurface());
+    QVERIFY(!s.isNull());
+    QVERIFY(s->isValid());
+    QSignalSpy surfaceCreatedSpy(m_compositorInterface, &CompositorInterface::surfaceCreated);
+    QVERIFY(surfaceCreatedSpy.isValid());
+    QVERIFY(surfaceCreatedSpy.wait());
+    auto serverSurface = surfaceCreatedSpy.first().first().value<SurfaceInterface*>();
+    QVERIFY(serverSurface);
+
+    // destroy client
+    QSignalSpy clientDisconnectedSpy(serverSurface->client(), &ClientConnection::disconnected);
+    QVERIFY(clientDisconnectedSpy.isValid());
+    QSignalSpy surfaceDestroyedSpy(serverSurface, &QObject::destroyed);
+    QVERIFY(surfaceDestroyedSpy.isValid());
+    if (m_connection) {
+        m_connection->deleteLater();
+        m_connection = nullptr;
+    }
+    QVERIFY(clientDisconnectedSpy.wait());
+    QCOMPARE(clientDisconnectedSpy.count(), 1);
+    QCOMPARE(surfaceDestroyedSpy.count(), 0);
+    QVERIFY(surfaceDestroyedSpy.wait());
+    QCOMPARE(surfaceDestroyedSpy.count(), 1);
+
+    s->destroy();
+    m_shm->destroy();
+    m_compositor->destroy();
+    m_queue->destroy();
+}
+
 QTEST_GUILESS_MAIN(TestWaylandSurface)
 #include "test_wayland_surface.moc"
