@@ -21,6 +21,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #define WAYLAND_SHELL_H
 
 #include <QObject>
+#include <QPoint>
 #include <QSize>
 
 #include <KWayland/Client/kwaylandclient_export.h>
@@ -36,6 +37,7 @@ namespace Client
 class EventQueue;
 class ShellSurface;
 class Output;
+class Seat;
 class Surface;
 
 /**
@@ -87,14 +89,14 @@ public:
      * Destroys the data held by this Shell.
      * This method is supposed to be used when the connection to the Wayland
      * server goes away. Once the connection becomes invalid, it's not
-     * possible to call release any more as that calls into the Wayland
+     * possible to call release anymore as that calls into the Wayland
      * connection and the call would fail. This method cleans up the data, so
-     * that the instance can be deleted or setup to a new wl_shell interface
+     * that the instance can be deleted or set up to a new wl_shell interface
      * once there is a new connection available.
      *
      * It is suggested to connect this method to ConnectionThread::connectionDied:
      * @code
-     * connect(connection, &ConnectionThread::connectionDied, shell, &Shell::destroyed);
+     * connect(connection, &ConnectionThread::connectionDied, shell, &Shell::destroy);
      * @endcode
      *
      * Right before the data is destroyed, the signal interfaceAboutToBeDestroyed is emitted.
@@ -149,6 +151,16 @@ Q_SIGNALS:
      **/
     void interfaceAboutToBeDestroyed();
 
+    /**
+     * The corresponding global for this interface on the Registry got removed.
+     *
+     * This signal gets only emitted if the Compositor got created by
+     * Registry::createShell
+     *
+     * @since 5.5
+     **/
+    void removed();
+
 private:
     class Private;
     QScopedPointer<Private> d;
@@ -185,12 +197,12 @@ public:
      **/
     void release();
     /**
-     * Destroys the data hold by this ShellSurface.
+     * Destroys the data held by this ShellSurface.
      * This method is supposed to be used when the connection to the Wayland
-     * server goes away. If the connection is not valid any more, it's not
-     * possible to call release any more as that calls into the Wayland
+     * server goes away. If the connection is not valid anymore, it's not
+     * possible to call release anymore as that calls into the Wayland
      * connection and the call would fail. This method cleans up the data, so
-     * that the instance can be deleted or setup to a new wl_shell_surface interface
+     * that the instance can be deleted or set up to a new wl_shell_surface interface
      * once there is a new connection available.
      *
      * This method is automatically invoked when the Shell which created this
@@ -213,8 +225,48 @@ public:
      **/
     void setFullscreen(Output *output = nullptr);
     void setMaximized(Output *output = nullptr);
+    void setToplevel();
+    /**
+     * Flags which can be passed to a transient surface.
+     * @see setTransient
+     * @since 5.5
+     **/
+    enum class TransientFlag {
+        Default = 0x0, ///< Default: transient surface accepts keyboard focus
+        NoFocus = 0x1 ///< Transient surface does not accept keyboard focus
+    };
+    Q_DECLARE_FLAGS(TransientFlags, TransientFlag)
+    /**
+     * Sets this Surface as a transient for @p parent.
+     *
+     * @param parent The parent Surface of this surface
+     * @param offset The offset of this Surface in the parent coordinate system
+     * @param flags The flags for the transient
+     * @since 5.5
+     **/
+    void setTransient(Surface *parent, const QPoint &offset = QPoint(), TransientFlags flags = TransientFlag::Default);
 
     bool isValid() const;
+
+    /**
+     * Requests a move on the given @p seat after the pointer button press with the given @p serial.
+     *
+     * @param seat The seat on which to move the window
+     * @param serial The serial of the pointer button press which should trigger the move
+     * @since 5.5
+     **/
+    void requestMove(Seat *seat, quint32 serial);
+
+    /**
+     * Requests a resize on the given @p seat after the pointer button press with the given @p serial.
+     *
+     * @param seat The seat on which to resize the window
+     * @param serial The serial of the pointer button press which should trigger the resize
+     * @param edges A hint for the compositor to set e.g. an appropriate cursor image
+     * @since 5.5
+     **/
+    void requestResize(Seat *seat, quint32 serial, Qt::Edges edges);
+
     operator wl_shell_surface*();
     operator wl_shell_surface*() const;
 
@@ -234,4 +286,7 @@ private:
 }
 }
 
+Q_DECLARE_METATYPE(KWayland::Client::ShellSurface::TransientFlag)
+Q_DECLARE_METATYPE(KWayland::Client::ShellSurface::TransientFlags)
+
 #endif
