@@ -18,7 +18,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "keyboard_interface.h"
-#include "resource_p.h"
+#include "keyboard_interface_p.h"
 #include "display.h"
 #include "seat_interface.h"
 #include "surface_interface.h"
@@ -36,32 +36,44 @@ namespace KWayland
 namespace Server
 {
 
-class KeyboardInterface::Private : public Resource::Private
-{
-public:
-    Private(SeatInterface *s, wl_resource *parentResource, KeyboardInterface *q);
-    void sendKeymap();
-    void sendKeymap(int fd, quint32 size);
-    void sendModifiers();
-    void sendModifiers(quint32 depressed, quint32 latched, quint32 locked, quint32 group, quint32 serial);
-
-    SeatInterface *seat;
-    SurfaceInterface *focusedSurface = nullptr;
-    QMetaObject::Connection destroyConnection;
-
-private:
-    // since version 3
-    static void releaseCallback(wl_client *client, wl_resource *resource);
-
-    static const struct wl_keyboard_interface s_interface;
-};
-
 KeyboardInterface::Private::Private(SeatInterface *s, wl_resource *parentResource, KeyboardInterface *q)
     : Resource::Private(q, s, parentResource, &wl_keyboard_interface, &s_interface)
     , seat(s)
 {
 }
 
+void KeyboardInterface::Private::focusChildSurface(const QPointer<SurfaceInterface> &childSurface, quint32 serial)
+{
+    if (focusedChildSurface == childSurface) {
+        return;
+    }
+    sendLeave(focusedChildSurface.data(), serial);
+    focusedChildSurface = childSurface;
+    sendEnter(focusedChildSurface.data(), serial);
+}
+
+void KeyboardInterface::Private::sendLeave(SurfaceInterface *surface, quint32 serial)
+{
+    if (surface && resource && surface->resource()) {
+        wl_keyboard_send_leave(resource, serial, surface->resource());
+    }
+}
+
+void KeyboardInterface::Private::sendEnter(SurfaceInterface *surface, quint32 serial)
+{
+    wl_array keys;
+    wl_array_init(&keys);
+    const auto states = seat->pressedKeys();
+    for (auto it = states.begin(); it != states.end(); ++it) {
+        uint32_t *k = reinterpret_cast<uint32_t*>(wl_array_add(&keys, sizeof(uint32_t)));
+        *k = *it;
+    }
+    wl_keyboard_send_enter(resource, serial, surface->resource(), &keys);
+    wl_array_release(&keys);
+
+    sendModifiers();
+}
+
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 const struct wl_keyboard_interface KeyboardInterface::Private::s_interface {
     releaseCallback
@@ -69,7 +81,7 @@ const struct wl_keyboard_interface KeyboardInterface::Private::s_interface {
 #endif
 
 KeyboardInterface::KeyboardInterface(SeatInterface *parent, wl_resource *parentResource)
-    : Resource(new Private(parent, parentResource, this), parent)
+    : Resource(new Private(parent, parentResource, this))
 {
 }
 
@@ -119,12 +131,9 @@ void KeyboardInterface::Private::sendModifiers()
 void KeyboardInterface::setFocusedSurface(SurfaceInterface *surface, quint32 serial)
 {
     Q_D();
-    if (d->focusedSurface) {
-        if (d->resource && d->focusedSurface->resource()) {
-            wl_keyboard_send_leave(d->resource, serial, d->focusedSurface->resource());
-        }
-        disconnect(d->destroyConnection);
-    }
+    d->sendLeave(d->focusedChildSurface, serial);
+    disconnect(d->destroyConnection);
+    d->focusedChildSurface.clear();
     d->focusedSurface = surface;
     if (!d->focusedSurface) {
         return;
@@ -133,20 +142,12 @@ void KeyboardInterface::setFocusedSurface(SurfaceInterface *surface, quint32 ser
         [this] {
             Q_D();
             d->focusedSurface = nullptr;
+            d->focusedChildSurface.clear();
         }
     );
+    d->focusedChildSurface = QPointer<SurfaceInterface>(surface);
 
-    wl_array keys;
-    wl_array_init(&keys);
-    const auto states = d->seat->pressedKeys();
-    for (auto it = states.begin(); it != states.end(); ++it) {
-        uint32_t *k = reinterpret_cast<uint32_t*>(wl_array_add(&keys, sizeof(uint32_t)));
-        *k = *it;
-    }
-    wl_keyboard_send_enter(d->resource, serial, d->focusedSurface->resource(), &keys);
-    wl_array_release(&keys);
-
-    d->sendModifiers();
+    d->sendEnter(d->focusedSurface, serial);
     d->client->flush();
 }
 
