@@ -105,14 +105,6 @@ void ShadowManagerInterface::Private::createShadow(wl_client *client, wl_resourc
         delete shadow;
         return;
     }
-    QObject::connect(s, &QObject::destroyed, shadow,
-        [shadow] {
-            if (shadow->resource()) {
-                wl_resource_destroy(shadow->resource());
-                delete shadow;
-            }
-        }
-    );
     s->d_func()->setShadow(QPointer<ShadowInterface>(shadow));
 }
 
@@ -368,10 +360,6 @@ ShadowInterface::Private::~Private()
     CURRENT(bottom)
     CURRENT(bottomLeft)
 #undef CURRENT
-    if (resource) {
-        wl_resource_destroy(resource);
-        resource = nullptr;
-    }
 }
 
 ShadowInterface::ShadowInterface(ShadowManagerInterface *parent, wl_resource *parentResource)
